clc; clear all;
close all
%% DC motors parameter estimation using Least Squares
load XmotorDataFriction
load YmotorDataFriction
load ZmotorDataFriction427g
Ts=0.01;

% Xmotor parameter derivation
Rmx=21.501; Kmx=30.593e-3; Kbx=Kmx; %Known parameters
rgx=1/76.84; Rpx=7.5e-2/2; Mx=3.868;
Kx=Kmx/Rmx;

Vaxf=SDxf.signals.values(:,2)*24;
xf0=SDxf.signals.values(1,1);
Vxf=[0;diff(SDxf.signals.values(:,1))/0.01];
Vxf1=[0;Vxf(1:end-1)];
Vaxf1=[0;Vaxf(1:end-1)];
phixf=[Vxf1 Vaxf1 sign(Vxf) ones(length(Vxf),1)];
thxf=inv(phixf'*phixf)*phixf'*Vxf; % subscript 'f' denotes the model param with friction

t1=SDxf.time;
a1xf=thxf(1);
a2xf=thxf(2);
a3xf=-thxf(3);
a4xf=-thxf(4);
Hxf=[(a1xf-1) a1xf*Ts;a2xf a2xf*Ts]^-1*[0;Ts*Kx];
Jxf=Hxf(1);
Bxf=Hxf(2);

betax1=(a3xf*(Jxf+Bxf*Ts))/Ts;
betax2=(a4xf*(Jxf+Bxf*Ts))/Ts;
alphaxf1=betax1+betax2;
alphaxf2=betax1-betax2;
%% 
% Ymotor parameter derivation
Rmy=21.501; Kmy=30.593e-3;Ky=Kmy/Rmy;
rgy=1/76.84; Rpy=7.5e-2/2; 

Vayf=SDyf.signals.values(:,4)*24;
Vyf=[0;diff(SDyf.signals.values(:,3))/0.01];
yf0=SDyf.signals.values(1,3);
Vyf1=[0;Vyf(1:end-1)];
Vayf1=[0;Vayf(1:end-1)];
phiyf=[Vyf1 Vayf1 sign(Vyf) ones(length(Vyf),1)];
thyf=inv(phiyf'*phiyf)*phiyf'*Vyf; % subscript 'f' denotes the model param with friction

a1yf=thyf(1);
a2yf=thyf(2);
a3yf=-thyf(3);
a4yf=-thyf(4);
Hyf=[(a1yf-1) a1yf*Ts;a2yf a2yf*Ts]^-1*[0;Ts*Ky];
Jyf=Hyf(1);
Byf=Hyf(2);

betay1=(a3yf*(Jyf+Byf*Ts))/Ts;
betay2=(a4yf*(Jyf+Byf*Ts))/Ts;
alphayf1=betay1+betay2;
alphayf2=betay1-betay2;
%%
% Zmotor parameter derivation
Rmz=21.501; Kmz=30.593e-3; Kbz=Kmz; % Known parameteres
rgz=1/76.84; Rpz=0.0851/(2*pi); m1=0.735; m2=0.427; g=9.81; 
% Real m is 734=~735g not 0.84kg. The weight of the bar is 44.21g
Kz=Kmz/Rmz;
Mz1=m1*g*Rpz*rgz; Mz2=m2*g*Rpz*rgz;

Vazf=SDzf1.signals.values(:,6)*24;
Vazfp=Vazf+Mz2/Kz;
lf0=SDzf1.signals.values(1,5);
Vzf=[0;diff(SDzf1.signals.values(:,5))/0.01];
Vzf1=[0;Vzf(1:end-1)];
Vazf1=[0;Vazfp(1:end-1)];
phizf=[Vzf1 Vazf1 sign(Vzf) ones(length(Vzf),1)];
thzf=inv(phizf'*phizf)*phizf'*Vzf;

a1zf=thzf(1);
a2zf=thzf(2);
a3zf=-thzf(3);
a4zf=-thzf(4);
Hzf=[(a1zf-1) a1zf*Ts;a2zf a2zf*Ts]^-1*[0;Ts*Kz];
Jzf=Hzf(1)-rgz*Rpz*m2;
Bzf=Hzf(2);

betaz1=(a3zf*(Jzf+Bzf*Ts))/Ts;
betaz2=(a4zf*(Jzf+Bzf*Ts))/Ts;
alphazf1=betaz1+betaz2;
alphazf2=betaz1-betaz2;
%%
J=[diag([Jxf Jyf Jzf]) zeros(3,2);zeros(2,3) zeros(2,2)];
B=[diag([Bxf Byf Bzf]) zeros(3,2);zeros(2,3) zeros(2,2)];
K=[diag([Kx Ky Kz]);zeros(2,3)];
R=[diag([rgx*Rpx rgy*Rpy rgz*Rpz]) zeros(3,2);zeros(2,3) eye(2)];
%% Discrete-model of the Motor+crane
GcVx=tf(Kx,[Jxf Bxf]);
GdVx=c2d(GcVx,Ts);
[Nx,Dx]=tfdata(GdVx);
GdisVx=tf(1,[Jxf Bxf]);
GdisdVx=c2d(GdisVx,Ts);
[Nxdis,Dxdis]=tfdata(GdisdVx);

GcVy=tf(Ky,[Jyf Byf]);
GdVy=c2d(GcVy,Ts);
[Ny,Dy]=tfdata(GdVy);
GdisVy=tf(1,[Jyf Byf]);
GdisdVy=c2d(GdisVy,Ts);
[Nydis,Dydis]=tfdata(GdisdVy);

GcVz=tf(Kz,[Jzf Bzf]);
GdVz=c2d(GcVz,Ts);
[Nz,Dz]=tfdata(GdVz);
GdisVz=tf(1,[Jzf Bzf]);
GdisdVz=c2d(GdisVz,Ts);
[Nzdis,Dzdis]=tfdata(GdisdVz);

ax1=-Dx{1,1}(2); bx1=Nx{1,1}(2); bdx1=Nxdis{1,1}(2);
ay1=-Dy{1,1}(2); by1=Ny{1,1}(2); bdy1=Nydis{1,1}(2);
al1=-Dz{1,1}(2); bl1=Nz{1,1}(2); bdl1=Nzdis{1,1}(2);


Ax=[1 Ts;0 ax1]; Bx=[0;bx1]; Wdx=[0;-bdx1]; Cx=[1 0];
Ay=[1 Ts;0 ay1]; By=[0;by1]; Wdy=[0;-bdy1]; Cy=[1 0];
Al=[1 Ts;0 al1]; Bl=[0;bl1]; Wdl=[0;-bdl1]; Cl=[1 0];

O22=zeros(2,2); O21=zeros(2,1);O12=zeros(1,2);
Ad=[Ax O22 O22;O22 Ay O22;O22 O22 Al]; 
Bd=[Bx O21 O21;O21 By O21;O21 O21 Bl];
Wd=[Wdx O21 O21;O21 Wdy O21;O21 O21 Wdl];
Cd=[Cx O12 O12;O12 Cy O12;O12 O12 Cl];
sysd=ss(Ad,Bd,Cd,0,Ts);

% reference model
Ar=[1 Ts;0 1]; Br=[0;Ts]; Cr=[1 0];
Am=[Ar O22 O22;O22 Ar O22;O22 O22 Ar]; 
Bm=[Br O21 O21;O21 Br O21;O21 O21 Br];
Cm=[Cr O12 O12;O12 Cr O12;O12 O12 Cr];
sysrefm=ss(Ar,Br,Cr,0,Ts);
%Swing Angle control Gain
kthx=0.17; kthy=0.17;
Kth=[kthx 0;0 kthy];

%Swing Angle Observer Gain
At=[0 1;0 0];
Ath=[At O22;O22,At];
Cth=[Cr O12;O12 Cr];
ep=0.01;
Ht=[2/ep;100/ep];
H=[Ht O21;O21 Ht];
eig(Ath-H*Cth)

At=[Ar O22;O22 Ar];Ct=Cth;
Hd1=acker(Ar',Cr',[0.5;0.5])';
eig(Ar-Hd1*Cr)
Hd=[Hd1 O21;O21 Hd1];
eig(At-Hd*Ct)

 m=m2; 
 %Friction correction
 alphayf2=alphayf2*1.5;
 alphaxf2=alphaxf2*1.15;
 alphaxf1=alphaxf1*1.32;
 
%% Servo Controller
% Kk=place(Ad,Bd,[0.62;0.88;0.73;0.74]);
% Kk(1,3:4)=O12; Kk(2,1:2)=O12;Kk;
Kkx=place(Ax,Bx,[0.79 0.89]);  %[0.86 0.66]
Kky=place(Ay,By,[0.78 0.63]);   %[0.8 0.65]
Kkl=place(Al,Bl,[0.73 0.74]);   %[0.75 0.7]
Kk=[Kkx O12 O12;O12 Kky O12;O12 O12 Kkl]

% Lk1=place(Ad',Cd',[0.61 0.87 0.63 0.66])';
% Lk1(3:4,1)=O21;Lk1(1:2,2)=O21;Lk1;
Lkx=place(Ax',Cx',[0.58 0.87])';  %[0.85 0.65]
Lky=place(Ay',Cy',[0.6 0.77])';   %[0.75 0.6]
Lkl=acker(Al',Cl',[0.6 0.65])';  %[0.65 0.65]
Lk=[Lkx O21 O21;O21 Lky O21;O21 O21 Lkl]

Phix=[0 (1-ax1)/bx1]; gamax=Ts/bx1; landax=bdx1/bx1;
Phiy=[0 (1-ay1)/by1]; gamay=Ts/by1; landay=bdy1/by1;
Phil=[0 (1-al1)/bl1]; gamal=Ts/bl1; landal=bdl1/bl1;
Phiff=[Phix O12 O12;O12 Phiy O12;O12 O12 Phil];
Gamaff=[gamax 0 0;0 gamay 0;0 0 gamal];
Landaff=[landax 0 0;0 landay 0;0 0 landal];

q_filter=[1 0 0 0 0 0;0 0 1 0 0 0;0 0 0 0 1 0];

%% Trajectory design
thx0=0; thy0=0;
 
%Vmaxx=0.34 Vmx=0.2 ==> accx=0.1 for minimum-time trajectory 
%accx=0.0225; tb=4; tfi=9; x0=0.05; xf=0.5; %(xf-x0=Vmax*(tf-tb)  acc=Vmax/tb)
%accx=0.05; tb=2; tfi=5; x0=0.05; xf=0.35;
accx=0.075; tb=2; tfi=5; x0=0.05; xf=0.5;
t=(0:Ts:tfi)';

qxddt=[accx*ones(length(t(1:tb/Ts)),1);...
      zeros(length(t(tb/Ts+1:(tfi-tb)/Ts)),1);...
      -accx*ones(length(t((tfi-tb)/Ts+1:tfi/Ts)),1)];
  
tr=4; % the pause time till the next repeat

qxddtt=[qxddt;zeros(tr/Ts,1);-qxddt;zeros(tr/Ts,1)];
%Uthx_f=[ones(tfi/Ts,1);zeros(tr/Ts,1);ones(tfi/Ts,1);zeros(tr/Ts,1)];
X_f=[x0*ones((tfi-tb)/Ts,1);xf*ones((tfi+tr)/Ts,1);x0*ones((tb+tr)/Ts,1)];

j=3; %number of repeats

qxdd=[]; X_filter=[];%Uthx_filter=[];%Vx_filter=[];  Va_filter=[];
for i=1:j
    qxdd=[qxdd;qxddtt];
%   Uthx_filter=[Uthx_filter;Uthx_f];
    X_filter=[X_filter;X_f];
    
end

qxdd=[qxdd;qxdd(end)];
%Uthx_filter=[Uthx_filter;Uthx_filter(end)];
X_filter=[X_filter;X_filter(end)];

%%
%Vmaxy=0.35 Vmy=0.2 ==> accy=0.1 for minimum-time trajectory 
%accy=0.0225; y0=0.05; yf=0.5;
accy=0.075; y0=0.05; yf=0.5;

qyddt=[accy*ones(length(t(1:tb/Ts)),1);...
      zeros(length(t(tb/Ts+1:(tfi-tb)/Ts)),1);...
      -accy*ones(length(t((tfi-tb)/Ts+1:tfi/Ts)),1)];

qyddtt=[qyddt;zeros(tr/Ts,1);-qyddt;zeros(tr/Ts,1)];
%Uthy_f=[ones(tfi/Ts,1);zeros(tr/Ts,1);ones(tfi/Ts,1);zeros(tr/Ts,1)];
Y_f=[y0*ones((tfi-tb)/Ts,1);yf*ones((tfi+tr)/Ts,1);y0*ones((tb+tr)/Ts,1)];

qydd=[]; Y_filter=[];%Uthy_filter=[];
for i=1:j
    qydd=[qydd;qyddtt];
    %Uthy_filter=[Uthy_filter;Uthy_f];
    Y_filter=[Y_filter;Y_f];
    
end

qydd=[qydd;qydd(end)];
%Uthy_filter=[Uthy_filter;Uthy_filter(end)];
Y_filter=[Y_filter;Y_filter(end)];
%%
%Vmaxz=0.13 in hoisting up, 0.15 in hoisting down,
%Vmz=0.1 ==> accz=0.05 for minimum-time trajectory 
%accz=-0.05; tf2z=2; tfz=4; l0=0.25; lf=0.05;
accz=-0.1; tf2z=1; tfz=2; l0=0.2; lf=0.1;
tz=(0:Ts:tfz)';

qzddt=[accz*ones(length(tz(1:tf2z/Ts)),1);...
     -accz*ones(length(tz(tf2z/Ts+1:tfz/Ts+1)),1)];
qzddt=[qzddt(1:end-1);zeros(length(t(tb/Ts+1:(tfi-tb)/Ts)),1);-qzddt];

qzddtt=[qzddt(1:end-1);zeros(tr/Ts,1);qzddt(1:end-1);zeros(tr/Ts+1,1)];

qzdd=[];
for i=1:j
    qzdd=[qzdd;qzddtt];
end
qzdd=qzdd(1:end-(j-1));
%%
Tfinal=2*j*(tfi+tr);
Tfi=(0:Ts:Tfinal)';
sim ReferenceGeneratorCrane3D_2014
qx=qx_qxd(:,1);
qxd=qx_qxd(:,2);
qy=qy_qyd(:,1);
qyd=qy_qyd(:,2);
qz=qz_qzd(:,1);
qzd=qz_qzd(:,2);